#@ MODIF B_SENSIBILITE_JDC Build  DATE 10/05/2006   AUTEUR MCOURTOI M.COURTOIS 
# -*- coding: iso-8859-1 -*-
#            CONFIGURATION MANAGEMENT OF EDF VERSION
# ======================================================================
# COPYRIGHT (C) 1991 - 2002  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY  
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY  
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR     
# (AT YOUR OPTION) ANY LATER VERSION.                                                  
#                                                                       
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT   
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF            
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU      
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.                              
#                                                                       
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE     
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,         
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.        
# ======================================================================


# RESPONSABLE GNICOLAS G.NICOLAS
"""
Cette classe pilote les modifications de jdc dans le cas des calculs de sensibilit.
Ces attributs sont :
jdc : le jdc  analyser. C'est l'objet obtenu aprs les vrifications d'usage et le
      traitement des macro-commandes ventuelles. Il n'est modifi par aucune des
      mthodes de cette classe.
commandes_sensibles : classe SENSIBILITE_COMMANDES_SENSIBLES contenant toutes les
                      informations pour faire les modifications.
                      ce sont les listes des commandes dclanchant le processus,
                      les commandes  driver, etc.
Attention : aucun test n'a t fait quand une commande DETRUIRE est incluse dans le JDC.
Cela posera des problmes si c'est la destruction d'un concept impact par la drivation.
"""
import copy

from Noyau.N_utils import prbanner
from B_SENSIBILITE_DERIVATION          import SENSIBILITE_DERIVATION
from B_SENSIBILITE_MEMO_NOM_SENSI      import SENSIBILITE_MEMO_NOM_SENSI
from B_SENSIBILITE_COMMANDES_SENSIBLES import SENSIBILITE_COMMANDES_SENSIBLES

class SENSIBILITE_JDC :
   """
   """
   def __init__(self,jdc):
      self.jdc = jdc
      self.commandes_sensibles = SENSIBILITE_COMMANDES_SENSIBLES()
#       print self.jdc
#       print self.commandes_sensibles
      if CONTEXT.debug :
        DEBUG = 1
      else :
        DEBUG = 0
      self.DEBUG = DEBUG
 
# ---------------------------------------------------------------------------
#           Mthodes lies aux tudes de sensibilit
# ---------------------------------------------------------------------------

   def is_sensible(self) :
      """
      Retourne deux entiers :
      . erreur : 0 : tout s'est bien pass
                 1 : une commande MEMO_NOM_SENSI est prsente dans le jdc
      . est_sensible : 1 : le jdc fait l'objet d'une tude de sensibilit
                       0 : pas de sensibilite dans ce jdc.
      Les paramtres concerns sont mmoriss. On a alors deux listes : les 'vrais' paramtres
      sensibles et les autres (champs theta par exemple)

      La technique est la suivante :
      . A priori, pas de sensibilit.
      . On parcourt toutes les tapes du jdc. Pour chacune d'elles :
       . Si l'tape est une dfinition de paramtre sensible, on mmorise la sd produite.
       . Si l'tape est une commande "sensible", on cherche dans ses donnes la prsence d'une sd
         reprsentant un paramtre sensible connu. Si oui, on dclare qu'il y a de la sensibilit
         et on mmorise la sd / paramtre sensible implique.
         
      """
#
      if self.DEBUG :
        prbanner("Recherche de la sensibilit du JDC : " + self.jdc.nom)
#
# 1. Rcupration des caractrisations des commandes sensibles
# 1.1. mot cl pilotant le calcul de sensibilit. C'est sa prsence dans les commandes "sensibles"
#      qui caractrisera un calcul avec sensibilit.
      mot_cle = self.commandes_sensibles.mot_cle
      if self.DEBUG :
        print "Mot cl pilotant le calcul de sensibilit = ", mot_cle

# 1.2. commande mmorisant les noms pour la sensibilit
      commande_memo_nom_sensi = self.commandes_sensibles.memo_nom_sensi
      if self.DEBUG :
        print "Commande mmorisant les noms pour la sensibilit = ", commande_memo_nom_sensi

# 1.3. liste des commandes de dfinition des paramtres sensibles
      l_commandes_defi_para_sensi = self.commandes_sensibles.get_l_commandes_defi_para_sensi()
      if self.DEBUG :
        print "Liste des commandes de dfinition des paramtres sensibles : ", l_commandes_defi_para_sensi

# 1.4. liste des commandes qui pilotent la sensibilit (ce sont les commandes principales de calcul
#      thermique ou mcanique)
      l_commandes_sensibles = self.commandes_sensibles.get_l_commandes_sensibles()
      if self.DEBUG :
        print "Liste des commandes qui pilotent la sensibilit : ", l_commandes_sensibles
#
# 2. Analyse des tapes du jdc
#
      l_classe_para_sensi = []
      l_param_sensible = []
      l_param_autres = []
      est_sensible = 0
      erreur = 0
#
      for etape in self.jdc.etapes:
#        print "\n. Etape : ",etape.nom
#
# 2.1. L'tape est une dfinition de paramtre sensible
#      On mmorise la classe correspondant  la sd produite.
#
        if etape.nom is commande_memo_nom_sensi :
          print "La commande ",commande_memo_nom_sensi," est interdite dans un jeu de commandes."
          erreur = 1
#
# 2.2. L'tape est une dfinition de paramtre sensible
#      On mmorise la classe correspondant  la sd produite.
#
        elif etape.nom in l_commandes_defi_para_sensi :
          erreur0 = self.commandes_sensibles.add_classe_para_sensi(etape.sd.__class__)
          l_classe_para_sensi = self.commandes_sensibles.get_classe_para_sensi()
#
# 2.3. L'tape est une commande qui ralise un calcul de sensibilit
#      On parcourt tous les mots-cls qui sont renseigns dans le jdc en examen
#      Tant que ce n'est pas le mot-cl de la sensibilit, on ne fait rien.
#
        elif etape.nom in l_commandes_sensibles :
#
          for child in etape.mc_liste :

#           L'un des mots-cls renseigns est celui de la sensibilit : 
            if child.nom == mot_cle :
#             On dclare etre en prsence d'un calcul sensible
              est_sensible = 1
              if self.DEBUG :
                print ".. Reprage de sensibilit dans la commande ",etape.nom

#             On rcupre la liste des arguments associs  ce mot-cl
              if type(child.valeur) in ( type(()),type([])):
                liste = child.valeur
              else:
                liste = [child.valeur]

#             Un argument peut etre de deux types :
#             . soit c'est un paramtre sensible stricto sensu ; on le sait en regardant
#               s'il fait partie des paramtres sensibles dfinis prcdemment dans le jdc
#             . soit c'est un autre type de paramtres de drivation (champ theta par exemple)
#             On parcourt donc tous les arguments pour faire le tri et les stocker dans
#             les 2 listes ad-hoc.
              for argu in liste :
                if self.DEBUG :
                  print ".... Paramtre sensible : ",argu.nom
                aux = 0
                for classe_para_sensi in l_classe_para_sensi :
                  if argu.__class__ is classe_para_sensi :
                    aux = 1
#
                if aux :
                  if argu not in l_param_sensible :
                    l_param_sensible.append(argu)
                else :  
                  if argu not in l_param_autres :
                    l_param_autres.append(argu)
#      
      self.l_param_sensible = l_param_sensible
      self.l_param_autres   = l_param_autres
#
      if self.DEBUG :
        if est_sensible :
          print "\nRcapitulatif :"  
          for l_param in [l_param_sensible,l_param_autres] :
            liste = []
            for param in l_param :
              liste.append((param.nom,param))
            if l_param is l_param_sensible :
              txt = "sensibles"
            else :
              txt = "autres"
            print "Liste des paramtres "+txt+" : ",liste  
          texte = "est"
        else :
          texte = "n'est pas"
        prbanner("Le JDC "+texte+" concern par la sensibilit.")
        print "\nCode de retour : ",erreur 
#
      return erreur, est_sensible
#
   def new_jdc(self) :
      """
      Construit un objet JDC copie de self augmente des commandes de drivation
      Retourne :
      . erreur : ==0 : tout s'est bien pass
                 !=0 : problme
      . le nouveau jdc
      """
      if self.DEBUG :
        prbanner("Cration d'un JDC driv  partir de : " + self.jdc.nom)
#
#      print "dans new_jdc : ",self.commandes_sensibles
#      print "l_param_sensible = ",self.l_param_sensible
#      print "l_param_autres = ",self.l_param_autres
# 1. Prparation
#
      if self.DEBUG :
        print ". 1. Prparation"
#
      erreur = 0
#
# 1.1. Rcupration des diffrentes listes de commandes impliques dans la drivation
#     l_commandes_a_deriver = liste des commandes  driver si un de leurs arguments l'a t
      l_commandes_a_deriver = self.commandes_sensibles.get_l_commandes_a_deriver()
#     l_commandes_a_deriver_ensemble = liste des commandes  driver dans tous les cas
      l_commandes_a_deriver_ensemble = self.commandes_sensibles.get_l_commandes_a_deriver_ensemble()
#     l_commandes_sensibles = liste des commandes qui pilotent la sensibilit
      l_commandes_sensibles = self.commandes_sensibles.get_l_commandes_sensibles()
#     l_commandes_poursuite = liste des commandes qui pilotent une reprise de calcul
      l_commandes_poursuite = self.commandes_sensibles.get_l_commandes_poursuite()
#     commande mmorisant les noms pour la sensibilit
      commande_memo_nom_sensi = self.commandes_sensibles.memo_nom_sensi
#       print "Liste des commandes sensibles          : ", l_commandes_sensibles
#       print "Liste des commandes  driver          : ", l_commandes_a_deriver
#       print "Liste des commandes  driver ensemble : ", l_commandes_a_deriver_ensemble
#       print "Liste des commandes de poursuite       : ", l_commandes_poursuite
#       print "Commandes mmorisant des noms          : ", commande_memo_nom_sensi
#
      mot_cle = self.commandes_sensibles.mot_cle
#
# 1.2. Cration de la classe qui mmorisera les noms des diffrentes sd impliques
#      dans la sensibilit. Elle est initialise avec la liste des sd par rapport auxquelles
#      on drive.
#
      if self.DEBUG :
        print ".. Cration d'une instance de la classe SENSIBILITE_MEMO_NOM_SENSI"
      memo_nom_sensi = SENSIBILITE_MEMO_NOM_SENSI(self.l_param_sensible+self.l_param_autres)
#       print "memo_nom_sensi.get_l_param_sensi() = ",memo_nom_sensi.get_l_param_sensi()

# 1.3. Cration de la classe qui contiendra les mthodes des drivations du jdc
#
      if self.DEBUG :
        print ".. Cration d'une instance de la classe SENSIBILITE_DERIVATION"
      derivation = SENSIBILITE_DERIVATION(self.jdc,memo_nom_sensi,commande_memo_nom_sensi,self.DEBUG)

# 1.4. Certaines commandes, DEFI_MATERIAU par exemple, peuvent apparaitre plusieurs fois dans le jdc.
#      Si l'une est concerne par un paramtre sensible, toutes doivent etre drives
#      par rapport  ce paramtre.
#      On indique ici pour chaque paramtre quelles commandes sont  driver d'office.
#
      if len (self.l_param_sensible) != 0 :
        for etape in self.jdc.etapes:
          if etape.isactif() :
            if etape.nom in l_commandes_a_deriver_ensemble :
              on_derive = 0
              l_sd_utilisees = etape.get_sd_utilisees()
              for param in self.l_param_sensible :
                for sd_u in l_sd_utilisees :
                  if sd_u is param :
                    erreur = memo_nom_sensi.add_commande(param,etape.nom)
                    if erreur : break
#
# 1.5. Le calcul est-il une reprise depuis un calcul prcdent ?
#      Il suffit d'analyser la premire commande active.
#
      if not erreur :
#
        poursuite = 0
        for etape in self.jdc.etapes:
          print etape.nom
          if etape.isactif() :
            if etape.nom in l_commandes_poursuite :
              poursuite = 1
            break
#
        if self.DEBUG :
          print "Poursuite : ",poursuite
#
# 2. Cration du nouveau JDC
#     Elle se fait avec les principaux arguments du jdc  modifier, en particulier le catalogue des commandes
#     En cas de poursuite, il faut charger le contexte pour connaitre les objets dja construits.
#
      if not erreur :
#
        if self.DEBUG :
          print ". 2. Cration du nouveau JDC"
#
        if poursuite :
          context_ini = self.jdc.g_context
        else :
          context_ini = None
#
        new_jdc = self.jdc.definition(cata=self.jdc.cata,appli=self.jdc.appli,procedure="#\n",context_ini=context_ini)
        new_jdc.actif_status=1
#
#### CD : Je suis oblige de reaffecter les attributs cpu au new_jdc. Ca ne me plait pas trop.
#### CD : il y aurait sans doute moyen de faire mieux (via __init__ du N_JDC)
        new_jdc.cpu_user=self.jdc.cpu_user
        new_jdc.cpu_syst=self.jdc.cpu_syst
#
        new_jdc.compile()
        if not new_jdc.cr.estvide(): 
          self.MESSAGE("ERREUR DE COMPILATION DANS ACCAS - INTERRUPTION")
          print ">> JDC.py : DEBUT RAPPORT"
          print new_jdc.cr
          print ">> JDC.py : FIN RAPPORT"
          new_jdc.supprime()
          return 1, None
#
        new_jdc.exec_compile()
        if not new_jdc.cr.estvide(): 
          self.MESSAGE("ERREUR A L'INTERPRETATION DANS ACCAS - INTERRUPTION")
          print ">> JDC.py : DEBUT RAPPORT"
          print new_jdc.cr
          print ">> JDC.py : FIN RAPPORT"
          new_jdc.supprime()
          return 1, None
#
        CONTEXT.set_current_step(new_jdc)
#   Update du contexte : ncessaire pour avoir les constantes (parameters)
#                        dans le new_jdc
        new_jdc.const_context=self.jdc.const_context
#
        if self.DEBUG :
          print "Le nouveau JDC est cr en tant qu'objet : ",new_jdc
#
# 3. On parcourt toutes les tapes du JDC initial
#    On ne s'occupe que des tapes actives et qui ne sont pas des commentaires
#
      if ( not erreur and self.DEBUG ) :
        print ". 3. On parcourt toutes les tapes du JDC initial"
#
      for etape in self.jdc.etapes :
        if self.DEBUG :
           print "\nAAAAAAAAAAAAAAAAA TRAITEMENT DE ",etape.nom," AAAAAAAAAAAAAAAAA"
        if etape.isactif() :
          if hasattr(etape,'sd') :
        # 3.1. Copie telle quelle de l'tape initiale
#            print "Lancement de la copie de ",etape.nom
            #on ne recopie pas l'etape (il ne faut oublier de lui donner comme parent le nouveau jdc : new_jdc)
            #new_etape = etape.full_copy(parent=new_jdc)
            new_etape = etape
#            print "Fin de la copie de ",etape.nom
            if self.DEBUG :
              print ".. L'tape a t recopie  l'identique."

# 3.2. On repre si c'est une commande sensible ou non :
#        . Si c'est une commande sensible, elle va produire les sd drives en son sein. Ces
#          sd doivent avoir t dclares dans MEMO_NOM_SENSI auparavant ; on
#          enregistrera donc la commande sensible aprs.
#        . Sinon, il faut l'enregistrer tout de suite pour que la sd produite soit
#          connue avant l'appel  MEMO_NOM_SENSI
            if etape.nom in l_commandes_sensibles :
              enregistrement_tardif = 1
            else :
              enregistrement_tardif = 0
              #new_jdc.register(new_etape)
              new_jdc.append_reset(new_etape)
              #print ". enregistrement immediat de  = ",etape.nom

# 3.3. Si l'tape produit une sd, on va la driver dans chacun des cas suivants :
#        . C'est la dfinition d'un paramtre sensible stricto sensu
#        . C'est la dfinition d'un paramtre sensible d'un autre type
#        . La commande est  driver obligatoirement ou car un de ses arguments
#          est lui-meme issu d'une drivation par rapport  un paramtre sensible
#        On remarque que ces cas sont exclusifs, d'o un filtrage avec la variable a_faire.
            if etape.sd :
              a_faire = 1
#              print "type(etape.definition.sd_prod) = ",type(etape.definition.sd_prod)

# 3.3.1. Drivation d'une dfinition de paramtre sensible
#
              for param in self.l_param_sensible :
                if param is etape.sd :
                  if self.DEBUG :
                    print ".. Lancement de la drivation"
                  erreur = derivation.derivation_para_sensi(etape,param,new_jdc)
                  if erreur : break
                  a_faire = 0

# 3.3.2. Drivation d'une dfinition de paramtre autre
#
              if a_faire :
                for param in self.l_param_autres :
                  if param is etape.sd :
                    if self.DEBUG :
                      print ".. Lancement de la drivation"
                    erreur = derivation.derivation_para_autre(etape,param,new_jdc)
                    if erreur : break
                    a_faire = 0

# 3.3.3. Drivation d'un autre type de commande
#          Remarques :
#          . Les seules commandes concernes sont les commandes sensibles ou  driver
#          . On rpte l'opration pour chacun des paramtres de drivation effectifs
              if a_faire :
                if etape.nom in l_commandes_sensibles + l_commandes_a_deriver + l_commandes_a_deriver_ensemble :
                  for param in self.l_param_sensible + self.l_param_autres :

# 3.3.3.1. Rcupration du dictionnaire des couples (nom simple,nom compos) existant pour ce paramtre
                    erreur,d_nom_s_c = memo_nom_sensi.get_d_nom_s_c(param)
                    if erreur : break
#
# 3.3.3.2. Doit-on effectivement driver par rapport au paramtre param ?
#         . Pour une commande du type ' driver ensemble', on ne le fait que si l'une d'entre
#           elles est implique par le paramtre sensible.
#         . Pour une commande non ncessairement  driver, on recherche si parmi les sd utilises
#           par la commande, il y en a au moins une qui a t drive par rapport  param
#           Remarque : plusieurs sd peuvent rpondre  cela ; il suffit de dcrocher ds
#           qu'une a t trouve car la drivation les prendra toutes en compte
#
                    on_derive = 0
                    if etape.nom in l_commandes_a_deriver_ensemble :
                      erreur, l_commandes = memo_nom_sensi.get_l_commandes(param)
                      if erreur : break
                      if etape.nom in l_commandes :
                        on_derive = 1
                    else :
                      l_sd_utilisees = etape.get_sd_utilisees()
                      for sd_u in l_sd_utilisees :
                        for sd_d in d_nom_s_c.keys() :
                          if sd_u is sd_d :
                            on_derive = 1
                      if on_derive :
                        if etape.nom not in l_commandes_sensibles :
                          daux = etape.get_sd_mcs_utilisees()
                          if self.DEBUG :
                            print ".. Dictionnaire des sd utilises : ", daux
                          if daux.has_key(mot_cle) : on_derive = 0
# 3.3.3.3. cas particulier CREA_CHAMP avec des mot-cls renseigns en dur : supprimer !
#
# 3.3.3.4. On doit driver par rapport au paramtre param ; alors on y va ...
                    if on_derive :
# 3.3.3.4.1. Si la commande en elle-meme est  driver, on y va
                      if etape.nom in l_commandes_a_deriver + l_commandes_a_deriver_ensemble :
                        if self.DEBUG :
                          print ".. Lancement de la drivation par derivation_commande"
                        erreur = derivation.derivation_commande(etape,param,new_jdc,d_nom_s_c)
                        if erreur : break
                      else :   
# 3.3.3.4.2. Sinon, il sufit de crer et de mmoriser un nouveau nom de sd. On commence par vrifier que cela
#            n'a pas dj t fait. Cela arrive quand un oprateur est appel plusieurs fois de suite.
#            Quand le nom est nouveau, il faut l'enregistrer dans le JDC via la commande MEM0_NON_SENSI
                        if self.DEBUG :
                          print ".. Recherche d'un nouveau nom compos"
                        erreur, nom_sd_derivee = memo_nom_sensi.get_nom_compose(etape.sd,param)
                        if erreur == 2 : 
                          nom_sd_derivee = derivation.get_nouveau_nom_sd()
                          erreur = memo_nom_sensi.add_nom_compose(etape.sd,param,nom_sd_derivee)
                          if erreur : break
#                          print "... code de retour de memo_nom_sensi.add_nom_compose = ",erreur
                          if erreur == 0 : 
                            txt = derivation.get_texte_memo_nom_sensi_compose(etape.sd.nom,param.nom,nom_sd_derivee,None)
                            exec txt in new_jdc.g_context

# 3.3.4. Enregistrement final si cela n'a pas t fait
            if enregistrement_tardif :
                #new_jdc.register(new_etape)
                new_jdc.append_reset(new_etape)
                #print ". enregistrement tardif de  = ",etape.nom
#
# 3.4. Message ventuel
          if erreur :
            print ".. Probleme dans la drivation de ",etape.nom
            break
          if self.DEBUG :
            print ".. Fin du traitement avec erreur = ", erreur
#
# 4. Dans le cas de poursuite, il faut dire que les bases ont dj t ouvertes
#
      if not erreur :
#
        if self.DEBUG :
          print ". 4. Cas de poursuite"
#
        if poursuite :
          new_jdc.ini = 1
#
        CONTEXT.unset_current_step()            
#
# 5. Fin
#
      if self.DEBUG :
        print "\n. 5. Nouveau JDC"
        for etape in new_jdc.etapes:
          print "Etape : ",etape.nom
        print "\n. "
        print "\nCode de retour : ",erreur 
        prbanner("Fin de la cration du JDC driv.")
#
      return erreur, new_jdc
